@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2007-2008  QRW Software
+--                 Copyright (C) 2007-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,12 @@
 --
 --
 -- $Log: add_password.c,v $
+-- Revision 1.13  2009/03/13 12:06:46  jsedwards
+-- Added include of objectify.h to define nwos_create_root function.
+--
+-- Revision 1.12  2009/03/08 00:06:17  jsedwards
+-- Changed include objectify_private.h to disk_io.h.
+--
 -- Revision 1.11  2008/09/01 18:25:56  jsedwards
 -- Changed to just verify there are chunks available and then call
 -- nwos_create_root which now does all of the heavy lifting.
@@ -73,8 +79,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "objectify_private.h"
-
+#include "disk_io.h"     /* define nwos_initialize_disk_io and nwos_terminate_disk_io */
+#include "objectify.h"   /* define nwos_create_root */
 
 
 int main(int argc, char* argv[])
