@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                    Copyright (C) 2007 QRW Software
+--                 Copyright (C) 2007-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -30,6 +30,13 @@
 --   into an adjacent block to free up that chunk.
 --
 -- $Log: eliminate_one_block_chunks.c,v $
+-- Revision 1.21  2008/09/01 00:22:59  jsedwards
+-- Fix year in copyright.  NO code changes.
+--
+-- Revision 1.20  2008/08/31 16:42:02  jsedwards
+-- Added assert around calls to nwos_read_object_from_disk because now it
+-- returns false if it fails instead of asserting itself.
+--
 -- Revision 1.19  2007/12/09 16:38:42  jsedwards
 -- Added code to modify the used_chunks in the header when copying file.
 --
@@ -348,7 +355,7 @@ static bool check_file(ObjRef* file_ref)
 
     memset(ivec, 0, sizeof(ivec));
 
-    nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj));  /* read the file object */
+    assert(nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj)));  /* read the file object */
 
     if (!is_void_reference(&file_obj.block_list))
     {
@@ -1124,7 +1131,7 @@ int main(int argc, char* argv[])
 
 	if (is_same_object(&object_class, &assoc_class_ref))
 	{
-	    nwos_read_object_from_disk(&ref_list->references[i], &assoc_obj, sizeof(assoc_obj));
+	    assert(nwos_read_object_from_disk(&ref_list->references[i], &assoc_obj, sizeof(assoc_obj)));
 
 	    for (j = 0; j < files_found; j++)
 	    {
