@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2007-2008  QRW Software
+--                 Copyright (C) 2007-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,12 @@
 --
 --
 -- $Log: list_public_spellings.c,v $
+-- Revision 1.9  2009/03/08 19:38:10  jsedwards
+-- Fixed year in Copyright.
+--
+-- Revision 1.8  2009/03/08 17:05:56  jsedwards
+-- Changed include objectify_private.h to objectify.h.
+--
 -- Revision 1.7  2008/09/01 00:23:06  jsedwards
 -- Fix year in copyright.  NO code changes.
 --
@@ -65,7 +71,7 @@
 #include <unistd.h>
 
 #include "../crc32.h"
-#include "../objectify_private.h"
+#include "../objectify.h"
 
 
 static size_t get_spelling_object_size(void* spelling_obj)
