@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2006-2008  QRW Software
+--                 Copyright (C) 2006-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,18 @@
 --
 --
 -- $Log: compress_sparse.c,v $
+-- Revision 1.29  2009/03/14 23:14:04  jsedwards
+-- Added include of the new chunk_info.h file.
+--
+-- Revision 1.28  2009/03/14 11:46:07  jsedwards
+-- Added include of user_config.h file.
+--
+-- Revision 1.27  2009/03/13 12:15:58  jsedwards
+-- Added includes for config.h, log.h, and progress_bar.h files.
+--
+-- Revision 1.26  2009/03/08 00:06:18  jsedwards
+-- Changed include objectify_private.h to disk_io.h.
+--
 -- Revision 1.25  2008/08/10 15:23:24  jsedwards
 -- Added path to "Missing magic number" and "Incorrect version in header" error
 -- messages.
@@ -155,7 +167,12 @@
 #include "gnu/md5.h"
 #include "gnu/sha1.h"
 
-#include "objectify_private.h"
+#include "chunk_info.h"
+#include "config.h"
+#include "disk_io.h"
+#include "log.h"
+#include "progress_bar.h"
+#include "user_config.h"
 
 
 static void print_usage(char *program)
