@@ -27,6 +27,12 @@
 --
 --
 -- $Log: disc.c,v $
+-- Revision 1.3  2009/03/13 12:21:34  jsedwards
+-- Added include of strlcxx.h file.
+--
+-- Revision 1.2  2009/03/08 00:13:54  jsedwards
+-- Changed include objectify_private.h to objectify.h.
+--
 -- Revision 1.1  2009/01/31 11:53:34  jsedwards
 -- Moved functions related to disc_lists and logging from file.c.
 --
@@ -44,7 +50,8 @@
 #include <unistd.h>
 
 #include "crc32.h"
-#include "objectify_private.h"
+#include "objectify.h"
+#include "strlcxx.h"           /* in case strlcpy and strlcat are not provided by the system */
 
 
 /*-------------------------------------------------------------------------------------------------------------------*/
