@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2006-2008  QRW Software
+--                 Copyright (C) 2006-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,16 @@
 --
 --
 -- $Log: import_file.c,v $
+-- Revision 1.44  2009/04/04 15:41:51  jsedwards
+-- Fix copyright years.  NO code changes.
+--
+-- Revision 1.43  2009/03/19 11:27:35  jsedwards
+-- Wrapped everything that was related to security levels with ifdef
+-- CHANGE_SECURITY_TO_DENSITY to disable it for the time being.
+--
+-- Revision 1.42  2009/03/14 03:31:24  jsedwards
+-- Added include of bit_map.h file.
+--
 -- Revision 1.41  2008/12/27 15:00:29  jsedwards
 -- Merged from alpha_29_5_branch back into main trunk.
 --
@@ -201,9 +211,12 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <time.h>
+
+#include "bit_map.h"
 #include "objectify.h"
 
 
+#ifdef CHANGE_SECURITY_TO_DENSITY
 struct options {
   Security_Level level;
   char* option;
@@ -219,14 +232,16 @@ struct options {
   };
 
 #define NUM_OPTIONS (sizeof(security_options)/sizeof(struct options))
-
+#endif
 
 static int result_counts[NUM_OBJ_RESULTS + 1];   /* allow one extra space for files that wouldn't fit */
 
 
 static void display_usage(char* prog_name)
 {
+#ifdef CHANGE_SECURITY_TO_DENSITY
     int i;
+#endif
 
     fprintf(stderr, "usage: %s [option] file [option] file...\n", prog_name);
     fprintf(stderr, "   or: %s --help\n", prog_name);
@@ -235,6 +250,7 @@ static void display_usage(char* prog_name)
     fprintf(stderr, "  --help\t\tdisplay this message.\n");
     fprintf(stderr, "  -r, --recursive\trecurse through directories.\n");
 
+#ifdef CHANGE_SECURITY_TO_DENSITY
     for (i = 0; i < NUM_OPTIONS; i++)
     {
 	fprintf(stderr, "  %s\tuse %s security level for following files.\n",
@@ -244,6 +260,7 @@ static void display_usage(char* prog_name)
     fprintf(stderr, "\n");
 
     fprintf(stderr, "See the file doc/Security_Levels.txt for more info on the security levels.\n");
+#endif
 
     fprintf(stderr, "\n");
 }
@@ -455,10 +472,16 @@ int main(int argc, char* argv[])
     {
 	if (argv[i][0] == '-')
 	{
+#ifdef CHANGE_SECURITY_TO_DENSITY
 	    for (j = 0; j < NUM_OPTIONS; j++)
 	    {
 		if (strcmp(argv[i], security_options[j].option) == 0) break;
 	    }
+#else
+#define NUM_OPTIONS 1234567890
+
+	    j = NUM_OPTIONS;
+#endif
 
 	    if (j == NUM_OPTIONS)
 	    {
@@ -529,12 +552,15 @@ int main(int argc, char* argv[])
 
     nwos_initialize_objectify(READ_WRITE, DEFAULT_FILE);
 
+#ifdef CHANGE_SECURITY_TO_DENSITY
     nwos_set_security_level(Security_Low);
-
+#endif
 
     for (i = 1; i < argc; i++)
     {
+#ifdef CHANGE_SECURITY_TO_DENSITY
 	nwos_restart_id_generation();   /* make sure each file starts with it's own sequence of id's */
+#endif
 
 	if (argv[i][0] == '-')
 	{
@@ -569,6 +595,7 @@ int main(int argc, char* argv[])
 	    }
 	    else
 	    {
+#ifdef CHANGE_SECURITY_TO_DENSITY
 		for (j = 0; j < NUM_OPTIONS; j++)
 		{
 		    if (strcmp(argv[i], security_options[j].option) == 0) break;
@@ -581,6 +608,7 @@ int main(int argc, char* argv[])
 		    nwos_set_security_level(security_options[j].level);
 		}
 		else
+#endif
 		{
 		    assert(strcmp(argv[i], "-r") == 0 || strcmp(argv[i], "--recursive") == 0);
 		}
