@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2006-2008  QRW Software
+--                 Copyright (C) 2006-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,13 @@
 --
 --
 -- $Log: log_disc.c,v $
+-- Revision 1.42  2009/04/04 15:41:51  jsedwards
+-- Fix copyright years.  NO code changes.
+--
+-- Revision 1.41  2009/03/19 11:38:21  jsedwards
+-- Wrapped call to nwos_set_security_level with ifdef CHANGE_SECURITY_TO_DENSITY
+-- because what was called security level stuff is temporarily disabled.
+--
 -- Revision 1.40  2008/09/01 03:13:43  jsedwards
 -- Change for new nwos_initialize_objectify calling convention (doesn't pass
 -- back root_object_reference anymore) and removed call to nwos_set_root_object
@@ -289,8 +296,9 @@ int main(int argc, char* argv[])
 
     nwos_initialize_objectify(READ_WRITE, DEFAULT_FILE);
 
+#ifdef CHANGE_SECURITY_TO_DENSITY
     nwos_set_security_level(Security_Very_Low);
-
+#endif
 
     if (!nwos_check_blocks_available(14))   /* 14 if new storage location is created, normally just 2 */
     {
