@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2004-2008  QRW Software
+--                 Copyright (C) 2004-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,56 @@
 --
 --
 -- $Log: objectify.c,v $
+-- Revision 1.195  2009/04/01 14:44:59  jsedwards
+-- Added call to check blocks available in create root so that last_ref is set.
+--
+-- Revision 1.194  2009/03/19 11:38:21  jsedwards
+-- Wrapped call to nwos_set_security_level with ifdef CHANGE_SECURITY_TO_DENSITY
+-- because what was called security level stuff is temporarily disabled.
+--
+-- Revision 1.193  2009/03/14 22:53:18  jsedwards
+-- Moved external declarations for nwos_update_file_001_object_to_current and
+-- nwos_update_file_002_object_to_current from the disk_io.h file.
+--
+-- Revision 1.192  2009/03/14 18:58:36  jsedwards
+-- Moved security related variables and functions to security.c and changed
+-- to call them there.
+--
+-- Revision 1.191  2009/03/14 14:35:55  jsedwards
+-- Deleted block_contains_object and nwos_object_exists functions, no longer
+-- needed.
+--
+-- Revision 1.190  2009/03/14 03:19:59  jsedwards
+-- Added includes for bit_map.h and objectify.h files.
+--
+-- Revision 1.189  2009/03/13 12:49:17  jsedwards
+-- Renamed old nwos_remove_object function that was in disk_io.c to
+-- nwos_erase_block and added nwos_remove_object function, that calls
+-- nwos_erase_block, to this file.
+--
+-- Revision 1.188  2009/03/11 12:28:40  jsedwards
+-- Moved nwos_malloc, nwos_realloc, and nwos_free functions to new mem_alloc.c
+-- file.
+--
+-- Revision 1.187  2009/03/08 19:53:46  jsedwards
+-- Fixed to compile correctly in PUBLIC_MODE again.
+--
+-- Revision 1.186  2009/03/08 06:34:10  jsedwards
+-- Changed include objectify_private.h to disk_io.h and added include of new
+-- class_definition.h file.  Moved global class reference variables to
+-- class_definition.c file.  Added new nwos_is_ready and nwos_is_writable
+-- functions.
+--
+-- Revision 1.185  2009/03/01 14:58:40  jsedwards
+-- Added nwos_realloc function.
+--
+-- Revision 1.184  2009/02/23 15:05:47  jsedwards
+-- Moved security_level, nwos_set_security_level, and nwos_generate_new_id to
+-- new gen_id.c file.
+--
+-- Revision 1.183  2009/02/17 01:51:35  jsedwards
+-- Fixed to compile correctly in PUBLIC_MODE.
+--
 -- Revision 1.182  2009/01/31 12:30:20  jsedwards
 -- Moved functions to deal with various file class definitions to
 -- class_definition.c file.
@@ -964,114 +1014,42 @@
 
 #include <assert.h>
 #include <ctype.h>
-#include <openssl/blowfish.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include "bit_map.h"
+#include "class_definition.h"
 #include "crc32.h"
-#include "objectify_private.h"
+#include "disk_io.h"
+#include "objectify.h"
+#include "security.h"
 #include "time_stamp.h"
 
 
+#define MAX_ROOT_TRANSFORMATIONS 128      // the number of different possible root objects for a given id
+
+
 /* Global variables... BAD!... need to fix */
 
 bool verbose = false;
 bool nwos_ready = false;
 
-ObjRef nwos_public_class_definition_class_ref;
-ObjRef nwos_private_class_definition_class_ref;
-ObjRef nwos_reference_list_class_ref;
+/* Defined in file.c, not for general usage */
 
-#ifndef PUBLIC_MODE
-static Security_Level security_level = Security_None;
-#endif
-
-#define SECURITY_VERY_LOW_N  7     /* sort of arbitrary, but it's always fun to use prime numbers */
-#define SECURITY_LOW_N      43     /* in this case each digit is represented */
-#define SECURITY_MEDIUM_N  509
-#define SECURITY_HIGH_N   2861
+extern void nwos_update_file_001_object_to_current(void* object, size_t size);
+extern void nwos_update_file_002_object_to_current(void* object, size_t size);
 
 
-#ifndef PUBLIC_MODE
-static uint32 linear_after_first;
-static uint32 serial_after_first;
-#endif
-
-uint8  nwos_random_sequence[NUM_STORED_SEQ][FILE_BLOCK_SIZE];     /* this will only work for blocks <= 256 bytes */
-
-#ifndef PUBLIC_MODE
-static BF_KEY blowfish_key;
-#endif
-
-
-
-/**************************************************************************/
-/* Special malloc and free routines that can help check for memory leaks. */
-/**************************************************************************/
-
-#ifdef CHECK_MEMORY_LEAKS
-#define MAX_MEM_ADDR_SAVED 16384
-void* save_malloc_addr[MAX_MEM_ADDR_SAVED];
-int cur_malloc_index;
-#endif
-
-void* nwos_malloc(size_t size)
+bool nwos_is_ready(void)
 {
-#ifdef CHECK_MEMORY_LEAKS
-    int i;
-#endif
-
-    void* result = malloc(size);
-
-    if (result == NULL)
-    {
-	char msg[64];
-	snprintf(msg, sizeof(msg), "%zd bytes", size);
-	perror(msg);
-	assert(result);
-	exit(1);
-    }
-
-#ifdef CHECK_MEMORY_LEAKS
-    for (i = 0; i < MAX_MEM_ADDR_SAVED; i++)
-    {
-	if (save_malloc_addr[cur_malloc_index] == NULL) break;
-	cur_malloc_index++;
-	if (cur_malloc_index >= MAX_MEM_ADDR_SAVED) cur_malloc_index = 0;  /* wrap if necessary */
-    }
-
-    assert(i < MAX_MEM_ADDR_SAVED);
-
-    save_malloc_addr[cur_malloc_index] = result;
-    cur_malloc_index++;
-    if (cur_malloc_index >= MAX_MEM_ADDR_SAVED) cur_malloc_index = 0;  /* wrap if necessary */
-#endif
-
-    return result;
+    return nwos_ready;
 }
 
-void nwos_free(void* ptr)
+bool nwos_is_writable(void)
 {
-#ifdef CHECK_MEMORY_LEAKS
-    int i;
-#endif
-
-    free(ptr);
-
-#ifdef CHECK_MEMORY_LEAKS
-    for (i = 0; i < MAX_MEM_ADDR_SAVED; i++)
-    {
-	cur_malloc_index--;
-	if (cur_malloc_index < 0) cur_malloc_index = MAX_MEM_ADDR_SAVED - 1;  /* wrap if necessary */
-	if (save_malloc_addr[cur_malloc_index] == ptr) break;
-    }
-
-    assert(i < MAX_MEM_ADDR_SAVED);
-
-    save_malloc_addr[cur_malloc_index] = NULL;    /* erase it */
-#endif
+    return nwos_disk_io_is_read_write();
 }
 
 
@@ -1127,43 +1105,6 @@ void nwos_encode_variable_sized_count(uint32 count, uint8 result[4])
 
 
 
-#ifdef DISABLE_SECURITY_FEATURES
-uint32 randomxxx;
-#endif
-
-
-/**********************/
-/* Sequence generator */
-/**********************/
-
-#ifndef PUBLIC_MODE
-static void generate_sequence_table(uint32* linear, uint32* serial, uint8 table[FILE_BLOCK_SIZE])
-{
-    int i;
-    uint8 j;
-    uint8 filled[FILE_BLOCK_SIZE];    /* remember which slots are already filled */
-
-    /* mark all slots as empty */
-    for (i = 0; i < FILE_BLOCK_SIZE; i++) filled[i] = 0;
-
-    /* then mark the first 8 as used */
-    for (i = 0; i < 8; i++) filled[i] = 1;
-
-    /* start at 0 because it is used already, so while loop will fail the first time. */
-    j = 0;
-    assert(filled[j] != 0);
-
-    for (i = 8; i < FILE_BLOCK_SIZE; i++)
-    {
-	/* find where the next byte is stored */
-	while(filled[j] != 0) j = nwos_next_sequence(linear, serial) % FILE_BLOCK_SIZE;
-	table[i] = j;   /* store it in the table */
-	filled[j] = 1;            /* mark it as used */
-    }
-}
-#endif
-
-
 /*********************************/
 /* Root object ID transformation */
 /*********************************/
@@ -1181,7 +1122,7 @@ static bool is_binary_number(uint n)
 }
 
 
-void nwos_transform_root(ObjRef* dst_ref, ObjRef* src_ref, int num)
+static void nwos_transform_root(ObjRef* dst_ref, ObjRef* src_ref, int num)
 {
     int shift_count;
 
@@ -1201,56 +1142,16 @@ void nwos_transform_root(ObjRef* dst_ref, ObjRef* src_ref, int num)
 /* Initialization and Termination */
 /**********************************/
 
-static void setup_from_big_key(uint8 big_key[16 + 8 + 4], ObjRef* root_object_ref)
-{
-    int i;
-    int j;
-    int k;
-    int tries;
-
-    BF_set_key(&blowfish_key, 16, big_key);
-
-    linear_after_first = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    serial_after_first = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    /* Create all of the sequence tables from the linear and serial parameters */
-    for (i = 0; i < NUM_STORED_SEQ; i++)
-    {
-	tries = 0;
-
-	do
-	{
-	    tries++;
-
-	    assert(tries < 4);
-
-	    generate_sequence_table(&linear_after_first, &serial_after_first, nwos_random_sequence[i]);
-
-	    for (j = 0; j < i; j++)
-	    {
-		for (k = 8; k < FILE_BLOCK_SIZE; k++) if (nwos_random_sequence[i][k] != nwos_random_sequence[j][k]) break;
-
-		if (k == FILE_BLOCK_SIZE) break;
-	    }
-	}
-	while (j < i);   /* if j is less than i it means one of the tables matched and needs to be regenerated */
-    }
-
-    assert(sizeof(ObjRef) == 4);   /* the following code assumes 4 byte IDs */
-
-    memcpy(root_object_ref->id, big_key+20, 4);
-}
-
-
 void nwos_initialize_objectify(AccessType type, const char* path)
 {
+    ObjRef root_object_ref;
+
 #ifdef PUBLIC_MODE
     assert(type == PUBLIC);
 #else
     int i;
     int attempt;  /* number of pass phrase attempts */
     uint8 big_key[16 + 8 + 4];
-    ObjRef root_object_ref;
     ObjRef test_object_ref;
     char* incorrect_messages[] = { "Incorrect, try again.", "Wrong again, one more try.", "Sorry, no go." };
 
@@ -1279,7 +1180,7 @@ void nwos_initialize_objectify(AccessType type, const char* path)
 	{
 	    nwos_get_key_from_password(big_key, sizeof(big_key));
 
-	    setup_from_big_key(big_key, &root_object_ref);
+	    nwos_setup_from_big_key(big_key, &root_object_ref);
 
 	    for (i = 0; i < MAX_ROOT_TRANSFORMATIONS; i++)
 	    {
@@ -1360,7 +1261,7 @@ void nwos_create_root(void)
 
 	nwos_get_key_from_password(big_key_1, sizeof(big_key_1));
 
-	setup_from_big_key(big_key_1, &test_object_ref);
+	nwos_setup_from_big_key(big_key_1, &test_object_ref);
 
 	random_transform = random() % MAX_ROOT_TRANSFORMATIONS;
 
@@ -1396,13 +1297,17 @@ void nwos_create_root(void)
 
     nwos_allocate_new_chunk(nwos_ref_to_word(&private_root_obj_ref));
 
+    nwos_check_blocks_available(3);
+
     hash = nwos_hash_ref(&private_root_obj_ref);
 
     assert(hash != 0 && !nwos_test_bit_in_map(hash));    /* make sure chunk is allocated, but block is not already used */
 
     nwos_set_bit_in_map(hash);
 
+#ifdef CHANGE_SECURITY_TO_DENSITY
     nwos_set_security_level(Security_Extreme);
+#endif
 
     nwos_clone_class_definition(&nwos_public_class_definition_class_ref, &nwos_private_class_definition_class_ref);
 
@@ -1476,7 +1381,9 @@ void nwos_terminate_objectify()
 
     nwos_ready = false;
 
+#ifndef PUBLIC_MODE
     nwos_flush_dirty_ref_lists();  /* flush any reference lists that are dirty out to disk */
+#endif
 
     nwos_terminate_disk_io();
 
@@ -1495,134 +1402,6 @@ bool nwos_in_public_mode()
 }
 
 
-/*******************************************************************/
-/* Reference utility functions: conversion, testing and generation */
-/*******************************************************************/
-
-bool block_contains_object(uint8 block[FILE_BLOCK_SIZE], ObjRef* ref)
-{
-    uint8 marker[8];
-
-    marker[0] = 0;
-    marker[1] = 0;
-    marker[2] = 0;
-    marker[3] = 0;
-    marker[4] = ref->id[0];
-    marker[5] = ref->id[1];
-    marker[6] = ref->id[2];
-    marker[7] = ref->id[3];
-
-    return (memcmp(block, marker, sizeof(marker)) == 0);   /* return true if the object marker is there */
-}
-
-
-bool nwos_object_exists(ObjRef* ref)
-{
-    uint8 block[FILE_BLOCK_SIZE];
-
-    return nwos_read_block(ref, block) && block_contains_object(block, ref);
-}
-
-
-#ifndef PUBLIC_MODE
-void nwos_set_security_level(Security_Level new_level)
-{
-    bool Security_Not_Set = false;
-
-    assert(Security_None < new_level && new_level <= Security_Extreme);
-
-    security_level = new_level;
-
-    switch (new_level)
-    {
-      case Security_Minimal:
-	nwos_set_allocation_parameters( ((USABLE_BLOCKS_PER_CHUNK * 7) / 8), 0, 1);
-	break;
-
-      case Security_Very_Low:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_VERY_LOW_N), 0, 4);
-	break;
-
-      case Security_Low:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_LOW_N), 0, 16);
-	break;
-
-      case Security_Medium:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_MEDIUM_N), 32, 32);
-	break;
-
-      case Security_High:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_HIGH_N), 128, 128);
-	break;
-
-      case Security_Extreme:
-	nwos_set_allocation_parameters(1, RESERVED_PRIVATE_CHUNKS, RESERVED_PRIVATE_CHUNKS);
-	break;
-
-      default:
-	assert(Security_Not_Set);
-	break;
-    }
-
-    nwos_restart_id_generation();   /* clear last_ref so this security level starts fresh */
-}
-#endif
-
-
-void nwos_generate_new_id(ObjRef* ref)
-{
-#ifdef PUBLIC_MODE
-    nwos_generate_new_public_id(ref);
-
-    assert(nwos_reference_type(ref) == Public_Reference);
-#else
-#if 0
-    bool Cant_Use_Security_None = false;
-#endif
-    bool Security_Not_Set = false;
-
-    switch (security_level)
-    {
-#if 0
-      case Security_None:
-	assert(Cant_Use_Security_None);
-	break;
-#endif
-
-      case Security_Minimal:
-	nwos_generate_new_closely_spaced_id(7, ref);
-	break;
-
-      case Security_Very_Low:
-	nwos_generate_new_1_in_N_id(SECURITY_VERY_LOW_N, ref);
-	break;
-
-      case Security_Low:
-	nwos_generate_new_1_in_N_id(SECURITY_LOW_N, ref);
-	break;
-
-      case Security_Medium:
-	nwos_generate_new_1_in_N_id(SECURITY_MEDIUM_N, ref);
-	break;
-
-      case Security_High:
-	nwos_generate_new_1_in_N_id(SECURITY_HIGH_N, ref);
-	break;
-
-      default:
-	assert(Security_Not_Set);
-	break;
-
-      case Security_Extreme:
-	nwos_generate_new_completely_random_id(ref);
-	break;
-    }
-
-    assert(nwos_reference_type(ref) == Private_Reference);
-#endif
-}
-
-
 /*************************/
 /* Verification routines */
 /*************************/
@@ -1672,118 +1451,9 @@ static bool check_object_data(ObjRef* ref, uint8* object, size_t total_size, uin
 }
 
 
-
-/*****************************************/
-/* Routines to read blocks, objects, etc */
-/*****************************************/
-
-bool nwos_read_object_from_disk_and_old_decrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
-{
-    uint8 encrypted[FILE_BLOCK_SIZE];
-#ifndef PUBLIC_MODE
-    uint8 scrambled[FILE_BLOCK_SIZE];
-    int i;
-    int j;
-#endif
-    bool result = false;
-
-    assert(!is_void_reference(ref));
-
-    if (nwos_read_block(ref, encrypted))
-    {
-	assert(block_contains_object(encrypted, ref));
-#ifdef PUBLIC_MODE
-	memcpy(object, encrypted, size);
-#else
-
-#ifdef DISABLE_SECURITY_FEATURES
-	if (true)
-#else
-	if (nwos_reference_type(ref) == Public_Reference)
-#endif
-	{
-	    memcpy(object, encrypted, size);    /* just copy the buffer into the object */
-	}
-	else
-	{
-	    /* copy the first 8 bytes directly, not encrypted */
-	    memcpy(object, encrypted, 8);
-
-	    /* decrypt the remaining bytes */
-	    BF_cbc_encrypt((encrypted + 8), (scrambled + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_DECRYPT);
-
-	    assert(size <= FILE_BLOCK_SIZE);
-
-	    for (i = 8; i < size; i++)
-	    {
-		j = seq_table[i];    /* find where the next byte is stored */
-		((uint8*)object)[i] = scrambled[j];    /* get this byte from there */
-	    }
-	}
-
-#endif
-	result = true;
-    }
-
-    return result;
-}
-
-
-bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
-{
-    uint8 scrambled[FILE_BLOCK_SIZE];
-#ifndef PUBLIC_MODE
-    uint8 encrypted[FILE_BLOCK_SIZE];
-    uint8 decrypted[FILE_BLOCK_SIZE];
-    int i;
-    int j;
-#endif
-    bool result = false;
-
-    assert(!is_void_reference(ref));
-
-    if (nwos_read_block(ref, scrambled))
-    {
-	assert(block_contains_object(scrambled, ref));
-#ifdef PUBLIC_MODE
-	memcpy(object, scrambled, size);
-#else
-
-#ifdef DISABLE_SECURITY_FEATURES
-	if (true)
-#else
-	if (nwos_reference_type(ref) == Public_Reference)
-#endif
-	{
-	    memcpy(object, scrambled, size);    /* just copy the buffer into the object */
-	}
-	else
-	{
-	    /* copy the first 8 bytes directly, not encrypted */
-	    memcpy(object, scrambled, 8);
-
-	    /* descramble */
-	    for (i = 8; i < FILE_BLOCK_SIZE; i++)
-	    {
-		j = seq_table[i];    /* find where the next byte is stored */
-		encrypted[i] = scrambled[j];    /* get this byte from there */
-	    }
-
-	    assert(size <= FILE_BLOCK_SIZE);
-
-	    /* decrypt */
-	    BF_cbc_encrypt((encrypted + 8), (decrypted + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_DECRYPT);
-
-	    memcpy((object + 8), (decrypted + 8), (size - 8));
-	}
-
-#endif
-	result = true;
-    }
-
-    return result;
-}
-
+/*********************************/
+/* Routines to read objects, etc */
+/*********************************/
 
 #define SIZE_EXTRA_BLOCK_DATA (FILE_BLOCK_SIZE - sizeof(uint32) - sizeof(ObjRef) - sizeof(ObjRef))
 
@@ -1825,7 +1495,7 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
     ptr_obj = object;
 
-    if (!nwos_read_object_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+    if (!nwos_read_block_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
 
     assert(sizeof(ObjRef) == 4);
 
@@ -1835,8 +1505,11 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
     if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
     {
+#ifdef PUBLIC_MODE
+	return false;
+#else
 	memset(ivec, 0, sizeof(ivec));
-	if (!nwos_read_object_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	if (!nwos_read_block_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
 
 	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
 	{
@@ -1844,6 +1517,7 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 	}
 
 	use_old_decryption = true;
+#endif
     }
 
     *ptr_obj++ = extra.flags[0];
@@ -1871,11 +1545,11 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
 	if (use_old_decryption)
 	{
-	    if (!nwos_read_object_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	    if (!nwos_read_block_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
 	}
 	else
 	{
-	    if (!nwos_read_object_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	    if (!nwos_read_block_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
 	}
 
     }
@@ -1956,7 +1630,7 @@ bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 
     ptr_obj = object;
 
-    if (!nwos_read_object_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+    if (!nwos_read_block_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
 
     assert(sizeof(ObjRef) == 4);
 
@@ -1966,8 +1640,11 @@ bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 
     if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
     {
+#ifdef PUBLIC_MODE
+	return false;
+#else
 	memset(ivec, 0, sizeof(ivec));
-	if (!nwos_read_object_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	if (!nwos_read_block_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
 
 	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
 	{
@@ -1975,6 +1652,7 @@ bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 	}
 
 	use_old_decryption = true;
+#endif
     }
 
     *ptr_obj++ = extra.flags[0];
@@ -2015,11 +1693,11 @@ bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 
 	if (use_old_decryption)
 	{
-	    if (!nwos_read_object_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	    if (!nwos_read_block_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
 	}
 	else
 	{
-	    if (!nwos_read_object_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	    if (!nwos_read_block_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
 	}
     }
 
@@ -2050,14 +1728,21 @@ void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header)
     assert(!is_void_reference(ref));
 
     memset(ivec, 0, sizeof(ivec));
-    assert(nwos_read_object_from_disk_and_decrypt(ref, header, sizeof(*header), ivec, nwos_random_sequence[0]));
+    assert(nwos_read_block_from_disk_and_decrypt(ref, header, sizeof(*header), ivec, 0));
 
     /* check_common_header(ref, &header->common); -- this currently doesn't do anything */
 
     if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
     {
+#ifdef PUBLIC_MODE
+	printf("\n");
+	fflush(stdout);
+	fprintf(stderr, "Unable to decrypt object headers: %08x\n", nwos_ref_to_word(ref));
+	nwos_terminate_objectify();
+	exit(1);
+#else
 	memset(ivec, 0, sizeof(ivec));
-	assert(nwos_read_object_from_disk_and_old_decrypt(ref, header, sizeof(*header), ivec, nwos_random_sequence[0]));
+	assert(nwos_read_block_from_disk_and_old_decrypt(ref, header, sizeof(*header), ivec, 0));
 
 	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
 	{
@@ -2067,6 +1752,7 @@ void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header)
 	    nwos_terminate_objectify();
 	    exit(1);
 	}
+#endif
     }
 
 #if 0
@@ -2081,63 +1767,9 @@ void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header)
 }
 
 
-/******************************************/
-/* Routines to write blocks, objects, etc */
-/******************************************/
-
-void nwos_write_object_to_disk_and_encrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
-{
-    uint8 scrambled[FILE_BLOCK_SIZE];
-#ifndef PUBLIC_MODE
-    uint8 decrypted[FILE_BLOCK_SIZE];
-    uint8 encrypted[FILE_BLOCK_SIZE];
-    int i;
-    int j;
-#endif
-
-    assert(!is_void_reference(ref));
-    assert(size <= FILE_BLOCK_SIZE);
-
-#ifdef PUBLIC_MODE
-    memset(scrambled, 0, FILE_BLOCK_SIZE);
-    memcpy(scrambled, object, size);
-#else
-
-#ifdef DISABLE_SECURITY_FEATURES    /* always do the unencryped code */
-    if (true)
-#else
-    if (nwos_reference_type(ref) == Public_Reference)
-#endif
-    {
-	memset(scrambled, 0, FILE_BLOCK_SIZE);    /* just copy the object into the buffer */
-	memcpy(scrambled, object, size);
-    }
-    else
-    {
-	/* copy the first 8 bytes straight across */
-	memcpy(scrambled, object, 8);
-
-	memcpy((decrypted + 8), (object + 8), size - 8);
-
-	/* fill any empty slots with random junk */
-	for (i = size; i < FILE_BLOCK_SIZE; i++) decrypted[i] = random();
-
-	/* and encrypt the remainder */
-	BF_cbc_encrypt((decrypted + 8), (encrypted + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_ENCRYPT);
-
-	for (i = 8; i < FILE_BLOCK_SIZE; i++)
-	{
-	    /* find where the next byte is stored */
-	    j = seq_table[i];    /* find where the next byte is stored */
-	    scrambled[j] = encrypted[i];    /* save this byte there */
-	}
-    }
-#endif
-
-    nwos_write_block(ref, scrambled);
-}
-
-
+/**********************************/
+/* Routines to write objects, etc */
+/**********************************/
 
 void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size)
 {
@@ -2195,7 +1827,7 @@ void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size)
 
 	nwos_generate_new_id(&extra.next_block);
 
-	nwos_write_object_to_disk_and_encrypt(&extra.id, &extra, sizeof(extra), ivec, nwos_random_sequence[seq]);
+	nwos_write_block_to_disk_and_encrypt(&extra.id, &extra, sizeof(extra), ivec, seq);
 
 	copy_reference(&extra.id, &extra.next_block);
 
@@ -2206,15 +1838,17 @@ void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size)
 
     for (i = 0; i < bytes_remaining - 8; i++) extra.data[i] = *ptr_obj++;
 
-    nwos_write_object_to_disk_and_encrypt(&extra.id, &extra, bytes_remaining, ivec, nwos_random_sequence[seq]);
+    nwos_write_block_to_disk_and_encrypt(&extra.id, &extra, bytes_remaining, ivec, seq);
 }
 
 
 void nwos_overwrite_object_to_disk(ObjRef* ref, void* object, size_t size)
 {
     uint8 ivec[IVEC_SIZE];
+    uint8 block[FILE_BLOCK_SIZE];
 
-    assert(nwos_object_exists(ref));
+    assert(size <= FILE_BLOCK_SIZE);        /* this function cannot handle objects larger than one block yet!! */
+    assert(nwos_read_block(ref, block));    /* verify that there is already an object there */
 
 #ifdef PUBLIC_MODE
     assert(nwos_reference_type(&((EveryObject*)object)->common.class_definition) == Public_Reference);
@@ -2225,7 +1859,19 @@ void nwos_overwrite_object_to_disk(ObjRef* ref, void* object, size_t size)
 
     memset(ivec, 0, sizeof(ivec));
 
-    nwos_write_object_to_disk_and_encrypt(ref, object, size, ivec, nwos_random_sequence[0]);
+    nwos_write_block_to_disk_and_encrypt(ref, object, size, ivec, 0);
+}
+
+
+void nwos_remove_object(ObjRef* ref)
+{
+    /* this function should read the object and erase all blocks, functions should not call this to erase a block, they should call erase_block */
+
+    bool this_needs_to_be_fixed_to_handle_multiple_block_objects = false;
+
+    assert(this_needs_to_be_fixed_to_handle_multiple_block_objects);
+
+    nwos_erase_block(ref);
 }
 
 
@@ -2264,7 +1910,7 @@ void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class)
 
     memset(ivec, 0, sizeof(ivec));
 
-    assert(nwos_read_object_from_disk_and_decrypt(obj, &header, sizeof(header), ivec, nwos_random_sequence[0]));
+    assert(nwos_read_block_from_disk_and_decrypt(obj, &header, sizeof(header), ivec, 0));
 
     /* check_common_header(obj, &header.common);  -- this doesn't currently do anything */
 
@@ -2274,8 +1920,15 @@ void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class)
     {
 	if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* could be old encryption */
 	{
+#ifdef PUBLIC_MODE
+	    printf("\n");
+	    fflush(stdout);
+	    fprintf(stderr, "Unable to decrypt object class without update: %08x\n", nwos_ref_to_word(obj));
+	    nwos_terminate_objectify();
+	    exit(1);
+#else
 	    memset(ivec, 0, sizeof(ivec));
-	    assert(nwos_read_object_from_disk_and_old_decrypt(obj, &header, sizeof(header), ivec, nwos_random_sequence[0]));
+	    assert(nwos_read_block_from_disk_and_old_decrypt(obj, &header, sizeof(header), ivec, 0));
 
 	    if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
 	    {
@@ -2288,6 +1941,7 @@ void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class)
 		    exit(1);
 		}
 	    }
+#endif
 	}
     }
 
