@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2004-2008  QRW Software
+--                 Copyright (C) 2004-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,50 @@
 --
 --
 -- $Log: objectify.h,v $
+-- Revision 1.165  2009/03/14 22:37:57  jsedwards
+-- Moved reference_list related declarations to reference_list.h file.
+--
+-- Revision 1.164  2009/03/14 15:21:25  jsedwards
+-- Moved nwos_get_key_from_password declaration to new security.h file.
+--
+-- Revision 1.163  2009/03/14 14:02:39  jsedwards
+-- Removed declaration for nwos_object_exists function which was deleted.
+--
+-- Revision 1.162  2009/03/14 11:16:45  jsedwards
+-- Moved nwos_check_blocks_available declaration to gen_id.h file.
+--
+-- Revision 1.161  2009/03/14 03:24:35  jsedwards
+-- Moved declaration for nwos_flush_bit_maps to new bit_map.h file.
+--
+-- Revision 1.160  2009/03/13 04:34:07  jsedwards
+-- Moved various declarations to disk_io.h, types.h, objects.h, reference.h,
+-- gen_id.h, strlcxx.h, log.h, reference_list.h, and progress_bar.h files.
+--
+-- Revision 1.159  2009/03/11 12:13:20  jsedwards
+-- Added include mem_alloc that I left out in previous check-in.
+--
+-- Revision 1.158  2009/03/11 12:03:23  jsedwards
+-- Moved nwos_malloc, nwos_realloc, and nwos_free external declarations to
+-- new mem_alloc.h file.
+--
+-- Revision 1.157  2009/03/10 12:12:55  jsedwards
+-- Moved the MAX_SPELLING_SIZE define from spelling.c, the MAX_NAME_SIZE from
+-- the name.c file, MAX_DIGITS, MAX_CARDINAL_NUMBER_OBJ_SIZE from the number.c
+-- file, MAX_NUMBER_SIZE, MAX_ID_SIZE, MAX_ACCOUNT_NUMBER_SIZE, MAX_CHECKING_SIZE
+-- from the financial.c file, MAX_URL_OBJ_SIZE from the url.c file and renamed.
+-- Also moved nwos_url_to_string, nwos_create_url, and
+-- nwos_create_public_manufacturer from the disk_io.h file.
+--
+-- Revision 1.156  2009/03/10 09:45:12  jsedwards
+-- Moved FILE_BLOCK_SIZE to the disk_io.h file.  Removed MAX_OBJECT_BLOCKS and
+-- MAX_OBJECT_SIZE because they were no longer used.  Added MAX_PATH_OBJ_SIZE
+-- to define the maximum size of a File Path object.
+--
+-- Revision 1.155  2009/03/08 01:58:39  jsedwards
+-- Moved object related declarations from disk_io.h file and added external
+-- declarations for new nwos_is_ready, nwos_is_writable,nwos_backup_is_enabled,
+-- and nwos_archive_version functions.
+--
 -- Revision 1.154  2008/12/27 15:00:29  jsedwards
 -- Merged from alpha_29_5_branch back into main trunk.
 --
@@ -749,205 +793,21 @@
 #define OBJECTIFY_H
 
 #include "config.h"
+#include "gen_id.h"
+#include "log.h"
+#include "mem_alloc.h"
+#include "objects.h"
+#include "progress_bar.h"
+#include "reference.h"
+#include "reference_list.h"
 #include "time_stamp.h"
 #include "types.h"
 
 
-#define FILE_BLOCK_SIZE 256
-
-#define MAGIC_NUMBER "NWOS"
-
-#define DEFAULT_FILE NULL      /* this used to define a path, now a null tells disk_io to use the default */
-
-typedef enum {
-  PUBLIC     = 0x50424C43,
-  READ_ONLY  = 0x52454144,
-  READ_WRITE = 0x52445752,
-} AccessType;
-
-
-typedef union { uint8 id[4]; uint32 word; } ObjRef;
-
-
-typedef struct {
-    uint32    flags;                      /*   4 */  /* flags / version */
-    ObjRef    id;                         /*   8 */  /* reference id */
-    union {
-      uint8     header_chksum[4];         /*  12 */  /* CRC32 of header */
-      uint32    num_refs;                            /* used by reference lists only */
-    };
-    uint8     data_chksum[4];             /*  16 */  /* CRC32 of data   */
-    TimeStamp creation_time;              /*  24 */
-    TimeStamp access_time;                /*  32 */  /* used when in memory for caching */
-    ObjRef    class_definition;           /*  36 */
-} CommonHeader;
-
-
-typedef struct {
-    ObjRef    clone_of;                   /*   4 */
-    ObjRef    references;                 /*   8 */
-    ObjRef    context;                    /*  12 */
-    ObjRef    creation;                   /*  16 */
-    ObjRef    source;                     /*  20 */
-    ObjRef    prev_version;               /*  24 */
-    ObjRef    next_version;               /*  28 */
-} ObjectHeader;
-
-
-typedef struct {
-    CommonHeader common;                  /*  36 */
-    ObjectHeader object;                  /*  64 */
-} EveryObject;
-
-typedef struct {
-    CommonHeader common_header;           /*  36 */
-    ObjRef       references[0];
-} ReferenceList;
-
-
-typedef struct {
-    CommonHeader  header;                 /*   36 */
-    uint32        index[256];             /*  ??? */
-    ObjRef        references[256];        /* ???? */
-} IndexedReferenceList;
-
-
-/* If we are making the export_c_structs program, c_structs.h doesn't   */
-/* exist yet, so these need to be defined for it to build successfully. */
-
-#ifdef USE_PREDEFINED_STRUCTS
-
-typedef struct {
-    EveryObject header;
-    ObjRef name;
-    uint8 count;
-    ObjRef feature[0];
-} C_struct_Class_Definition;
-
-typedef struct {
-    EveryObject header;
-    ObjRef character_set;
-    uint8 count;
-    char storage[0];
-} C_struct_Spelling;
-
-typedef struct {
-    EveryObject header;
-    uint8 count;
-    ObjRef spelling[0];
-} C_struct_Name;
-
-typedef struct {
-    EveryObject header;
-    ObjRef inherit;
-    ObjRef class;
-    ObjRef label;
-    uint8 count;
-} C_struct_Feature_Definition;
-
-typedef struct {
-    EveryObject header;
-    ObjRef class_definition_class;
-    ObjRef reference_list_class;
-} C_struct_Root;
-
-#else
-#include "c_structs.h"
-#endif
-
-
-static inline bool is_void_reference(ObjRef* ref)
-{
-    int i;
-    for (i = 0; i < sizeof(ObjRef); i++) if (ref->id[i] != 0) return false;
-    return true;
-}
-
-
-static inline bool is_same_object(ObjRef* ref1, ObjRef* ref2)
-{
-    int i;
-    for (i = 0; i < sizeof(ObjRef); i++) if (ref1->id[i] != ref2->id[i]) return false;
-    return true;
-}
-
-
-static inline void copy_reference(ObjRef* dst, ObjRef* src)
-{
-    int i;
-    for (i = 0; i < sizeof(ObjRef); i++) dst->id[i] = src->id[i];
-}
-
-
-static inline void void_reference(ObjRef* ref)
-{
-    int i;
-    for (i = 0; i < sizeof(ObjRef); i++) ref->id[i] = 0;
-}
-
-
-static inline void nwos_4_uint8_to_uint32(uint8* src, uint32* dst) 
-{
-    *dst = ((uint32)src[0] << 24) | ((uint32)src[1] << 16) | ((uint32)src[2] << 8) | (uint32)src[3];
-}
-
-static inline void nwos_2_uint8_to_uint16(uint8* src, uint16* dst) 
-{
-    *dst = ((uint32)src[0] << 8) | ((uint32)src[1]);
-}
-
-static inline void nwos_uint32_to_4_uint8(uint32* src, uint8* dst) 
-{
-    dst[0] = *src >> 24;
-    dst[1] = *src >> 16;
-    dst[2] = *src >> 8;
-    dst[3] = *src;
-}
-
-
-/* NOTE: these only work when references are 4 bytes, need to fix when size is changed */
-static inline uint32 nwos_ref_to_word(ObjRef* ref) 
-{
-  return ((uint32)ref->id[0] << 24) | ((uint32)ref->id[1] << 16) | ((uint32)ref->id[2] << 8) | (uint32)ref->id[3];
-}
-
-static inline void nwos_word_to_ref(uint32 word, ObjRef* ref) 
-{
-    ref->id[0] = word >> 24;
-    ref->id[1] = word >> 16;
-    ref->id[2] = word >> 8;
-    ref->id[3] = word;
-}
-
-static inline uint16 byteswap_uint16(uint16 x)
-{
-    return (x << 8) | (x >> 8);
-}
-
-static inline uint32 byteswap_uint32(uint32 x)
-{
-  return (x << 24) |
-    ((x & 0x0000ff00) << 8) |
-    ((x & 0x00ff0000) >> 8) |
-    (x >> 24);
-}
-
-
-typedef enum { Unknown_Reference, Public_Reference, Private_Reference, Temporary_Reference } Reference_Type;
-typedef enum { Security_None=0, Security_Minimal, Security_Very_Low, Security_Low, Security_Medium, Security_High, Security_Extreme } Security_Level;
 
 typedef enum { CREATED_NEW, IMPORTED_DATA, CREATED_NEW_REVISION, FOUND_EXISTING, ERROR_OCCURRED, DUPLICATE_PATH, DUPLICATE_FILE, PATH_NOT_FOUND, MULTIPLE_FILES, CREATED_NEW_PATH_FOR_EXISTING_FILE, NUM_OBJ_RESULTS } ObjCreateResult;
 
 
-/* These functions aren't standard */
-#ifndef HAVE_STRLCPY
-extern size_t strlcpy(char* dst, const char* src, size_t size);
-#endif
-
-#ifndef HAVE_STRLCAT
-extern size_t strlcat(char* dst, const char* src, size_t size);
-#endif
-
 extern void nwos_ref_to_name(ObjRef* ref, char name[]);
 
 /* extern ObjRef store_object(EveryObject* obj, uint64 length); */
@@ -956,45 +816,20 @@ extern uint32 nwos_decode_variable_sized_count(uint8 count[4]);
 extern void   nwos_encode_variable_sized_count(uint32 count, uint8 result[4]);
 
 
-#define MD5_DIGEST_SIZE    16
-#define SHA1_DIGEST_SIZE   20
-#define SHA256_DIGEST_SIZE 32
-#define SHA512_DIGEST_SIZE 64
-
-extern void nwos_log(char* str);
-extern void nwos_log_arguments(int argc, char* argv[]);
-extern void nwos_log_md5sum(const char* file_name, uint8 md5_digest[MD5_DIGEST_SIZE]);
-extern void nwos_log_sha1sum(const char* file_name, uint8 sha1_digest[SHA1_DIGEST_SIZE]);
-extern void nwos_log_sha256sum(const char* file_name, uint8 sha256_digest[SHA256_DIGEST_SIZE]);
-extern void nwos_log_sha512sum(const char* file_name, uint8 sha512_digest[SHA512_DIGEST_SIZE]);
-
 extern void nwos_initialize_objectify(AccessType type, const char* path);
 extern void nwos_terminate_objectify();
+extern void nwos_create_root(void);
 extern bool nwos_set_root_object(ObjRef* ref);
-extern void nwos_flush_bit_maps();
 extern bool nwos_in_public_mode();
+extern bool nwos_is_ready(void);
+extern bool nwos_is_writable(void);
+extern bool nwos_backup_is_enabled(void);
+extern const char* nwos_archive_version(void);
 
-extern size_t nwos_reference_list_size(ObjRef* ref);
-extern ReferenceList* nwos_malloc_reference_list(ObjRef* ref);
-extern void nwos_free_reference_list(ReferenceList* ref_list);
-
-extern bool nwos_is_public_reference(ObjRef* ref);
-extern bool nwos_is_private_reference(ObjRef* ref);
-extern bool nwos_is_temporary_reference(ObjRef* ref);
-extern Reference_Type nwos_reference_type(ObjRef* ref);
 extern void nwos_get_object_class(ObjRef* obj, ObjRef* object_class);
 extern void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class);
 extern size_t nwos_get_object_size(void* object);
-extern void nwos_restart_id_generation();
-extern void nwos_generate_new_id(ObjRef* ref);
-extern void nwos_generate_new_private_id(ObjRef* ref);
-extern bool nwos_check_blocks_available(uint32 estimated_blocks);
-extern void nwos_set_security_level(Security_Level);
-
-extern void nwos_add_to_references(ObjRef* ref, ObjRef* obj);
-extern void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list);
-extern void nwos_remove_from_references(ObjRef* ref, ObjRef* obj);
-extern void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list);
+extern void nwos_fill_in_common_header(CommonHeader* common, ObjRef* ref, ObjRef* class_definition_ref);
 
 //extern bool nwos_find_class_definition(char* name, ObjRef* class_ref);
 extern bool nwos_find_public_class_definition(char* name, ObjRef* class_ref);
@@ -1003,16 +838,21 @@ extern ObjCreateResult nwos_find_or_create_private_class_definition(char* name_o
 extern bool nwos_find_public_feature_definition(ObjRef* type_ref, char* name, int count, ObjRef* feat_ref);
 extern void nwos_find_or_create_public_feature_definition(ObjRef* type_ref, char* name, int count, ObjRef* feat_ref);
 
-extern void nwos_create_reference_list(ObjRef* for_obj, ObjRef* ref_list);
 extern void nwos_create_object(uint64 id, EveryObject* header);
 extern void nwos_read_class_definition(ObjRef* ref, C_struct_Class_Definition* class_def);
 extern void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header);
 extern bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size);
 extern bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* obj, size_t max_size, size_t (*size_function)(void*));
-extern void nwos_read_reference_list_from_disk(ObjRef* ref, ReferenceList* object, size_t size);
+extern void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size);
+extern void nwos_write_public_object_to_disk(ObjRef* ref, void* object, size_t size);
+extern void nwos_overwrite_object_to_disk(ObjRef* ref, void* object, size_t size);
+extern void nwos_remove_object(ObjRef* ref);
 
 
 /* word stuff */
+
+#define MAX_SPELLING_OBJ_SIZE  (sizeof(C_struct_Spelling) + 100)
+
 extern bool nwos_find_language(char* name, ObjRef* ref);
 
 extern bool nwos_find_public_spelling(char* spelling, ObjRef* ref);
@@ -1034,6 +874,9 @@ extern bool nwos_ask_yes_or_no(char* statement, char* question);
 
 
 /* name stuff */
+
+#define MAX_NAME_OBJ_SIZE  (sizeof(C_struct_Name) + sizeof(ObjRef) * 10)
+
 extern bool            nwos_find_public_name(const char* name, ObjRef* ref);
 extern bool            nwos_find_private_name(const char* name, ObjRef* ref);
 extern ObjCreateResult nwos_create_name(const char* name, ObjRef* ref);
@@ -1055,6 +898,11 @@ extern bool            nwos_month_number_to_string(ObjRef* month_obj_ref, char*
 extern bool            nwos_date_to_string(ObjRef* date_obj_ref, char* string, size_t size_of_string);
 
 /* numbers */
+
+#define MAX_CARDINAL_NUMBER_DIGITS 127
+
+#define MAX_CARDINAL_NUMBER_OBJ_SIZE (sizeof(C_struct_Cardinal_Number) + MAX_CARDINAL_NUMBER_DIGITS)
+
 extern bool            nwos_find_cardinal_number(char* number, ObjRef* ref);
 extern ObjCreateResult nwos_create_cardinal_number(char* number, ObjRef* ref);
 extern void            nwos_cardinal_number_to_string(ObjRef* ref, char* string, size_t size);
@@ -1090,10 +938,18 @@ extern bool nwos_find_state_from_name(char* name, ObjRef* ref);
 extern bool nwos_any_states_named(char* name);
 extern void nwos_describe_state(char* name);
 extern ObjCreateResult nwos_create_us_city(char* city, ObjRef* state_ref, ObjRef* city_ref);
+extern void nwos_private_create_city(ObjRef* city_class_ref, ObjRef* name_ref, ObjRef* state_ref, ObjRef* city_ref);
 extern void nwos_add_city(ObjRef* city_ref);
 extern void nwos_list_cities_in_state(char* state);
 
 /* financial stuff */
+
+#define MAX_ACCOUNT_NUMBER_SIZE 18
+#define MAX_CHECKING_ID_SIZE 4
+
+#define MAX_ACCOUNT_NUMBER_OBJ_SIZE  (sizeof(C_struct_Account_Number) + MAX_ACCOUNT_NUMBER_SIZE)
+#define MAX_CHECKING_OBJ_SIZE        (sizeof(C_struct_Checking) + MAX_CHECKING_ID_SIZE)
+
 extern bool nwos_find_public_credit_union(char* name, ObjRef* ref);
 extern void nwos_add_account();
 extern void nwos_list_accounts();
@@ -1105,9 +961,10 @@ extern void nwos_list_public_manufacturers();
 
 
 /* file stuff */
-typedef enum { File_Not_Found, Checksums_Match, MD5_Sum_Mismatch, SHA1_Sum_Mismatch, SHA256_Sum_Mismatch, SHA512_Sum_Mismatch, Checksums_Mismatch } CheckFileResult;
 
-extern bool nwos_url_to_string(ObjRef* ref, char* string, size_t size);
+#define MAX_PATH_OBJ_SIZE  (sizeof(C_struct_File_Path) + 4096)
+
+typedef enum { File_Not_Found, Checksums_Match, MD5_Sum_Mismatch, SHA1_Sum_Mismatch, SHA256_Sum_Mismatch, SHA512_Sum_Mismatch, Checksums_Mismatch } CheckFileResult;
 
 extern void nwos_list_public_credit_unions();
 
@@ -1158,19 +1015,18 @@ extern ObjCreateResult nwos_create_disc_copy(ObjRef* disc_list, int copy_num, Ob
 extern bool nwos_find_storage_location(const char* location, ObjRef* ref);
 extern ObjCreateResult nwos_create_storage_location(const char* location, ObjRef* ref);
 
+/* uniform resource locator stuff */
+
+#define MAX_URL_OBJ_SIZE   (sizeof(C_struct_Uniform_Resource_Locator) + 160)
+extern bool nwos_url_to_string(ObjRef* ref, char* string, size_t size);
+
+extern ObjCreateResult nwos_create_url(char* url, ObjRef* ref);
+
+extern void nwos_create_public_acronym(char* acronym, ObjRef* ref);
+
+extern void nwos_create_public_manufacturer(char* brand_name, char* acronym, char* url_list[], ObjRef* ref);
+
 extern bool nwos_is_valid_ssn(const char* ssn);
 
-extern void nwos_get_key_from_password(uint8 key[], size_t key_size);
-
-extern void nwos_start_progress_bar();
-extern void nwos_update_progress_bar(float percent);  /* 0.0 <= percent < 1.0 */
-extern void nwos_erase_progress_bar();
-extern void nwos_finish_progress_bar();
-
-
-/* currently the largest object is a disc list */
-#define MAX_OBJECT_BLOCKS ((MAX_SIZE_DISC_LIST + FILE_BLOCK_SIZE - 1) / FILE_BLOCK_SIZE)
-#define MAX_OBJECT_SIZE (MAX_OBJECT_BLOCKS * FILE_BLOCK_SIZE)
-
 #endif
 
