@@ -27,6 +27,29 @@
 --
 --
 -- $Log: reference_list.c,v $
+-- Revision 1.45  2009/03/14 20:25:52  jsedwards
+-- Fix bug introduced in previous check-in where seq was not set to block_num.
+--
+-- Revision 1.44  2009/03/14 19:28:34  jsedwards
+-- Added include of security.h file and renamed read and write object and
+-- encrypt or decrypt functions to read and write block.
+--
+-- Revision 1.43  2009/03/14 03:31:24  jsedwards
+-- Added include of bit_map.h file.
+--
+-- Revision 1.42  2009/03/13 13:10:15  jsedwards
+-- Added includes for log.h and reference_list.h files.  Also moved
+-- MAX_REFS_IN_REF_LIST, MAX_REFS_IN_SIDECAR, Ref_List_Extra_Block, and
+-- Ref_List_First_Block.
+--
+-- Revision 1.41  2009/03/08 19:21:49  jsedwards
+-- Surrounded nwos_remove_from_reference_list and nwos_remove_from_references
+-- with ifndef PUBLIC_MODE so they won't be compiled in public mode.
+--
+-- Revision 1.40  2009/03/08 00:52:42  jsedwards
+-- Changed include objectify_private.h to disk_io.h and added include
+-- class_definition.h.
+--
 -- Revision 1.39  2009/01/24 18:16:04  jsedwards
 -- Added test to nwos_remove_from_reference_list to verify the last_block
 -- pointer is valid after changing the reference list.  Also undefined
@@ -187,13 +210,40 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "bit_map.h"
+#include "class_definition.h"
 #include "crc32.h"
-#include "objectify_private.h"
+#include "disk_io.h"
+#include "log.h"
+#include "reference_list.h"
+#include "security.h"
 
 
 #undef VERIFY_LAST_BLOCK_POINTER   /* disable tests that verify the last block pointer is correct */
 
 
+#define MAX_REFS_IN_REF_LIST (((FILE_BLOCK_SIZE - sizeof(CommonHeader)) / sizeof(ObjRef)) - 1)
+#define MAX_REFS_IN_SIDECAR  (((FILE_BLOCK_SIZE - 12) / sizeof(ObjRef)) - 1)
+
+typedef struct ref_list_extra_block {
+    struct ref_list_extra_block* next_block_ptr;
+    int dirty;
+    ObjRef id;
+    uint8 checksum[4];
+    ObjRef refs[MAX_REFS_IN_SIDECAR];
+    ObjRef next_block_ref;
+    uint8 ivec[IVEC_SIZE];
+} Ref_List_Extra_Block;
+
+typedef struct {
+    struct ref_list_extra_block* next_block_ptr;
+    ReferenceList list;
+    ObjRef refs[MAX_REFS_IN_REF_LIST];
+    ObjRef next_block_ref;
+    uint8 ivec[IVEC_SIZE];
+} Ref_List_First_Block;
+
+
 #define REF_LIST_CACHE_SIZE 64
 
 typedef struct {
@@ -263,11 +313,11 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	if (use_old_decryption)
 	{
-	    assert(nwos_read_object_from_disk_and_old_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]));
+	    assert(nwos_read_block_from_disk_and_old_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, 0));
 	}
 	else
 	{
-	    assert(nwos_read_object_from_disk_and_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]));
+	    assert(nwos_read_block_from_disk_and_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, 0));
 	}
 
 	if (nwos_reference_type(&cache->ref) == Public_Reference || is_same_object(&cache->first_block->list.common_header.class_definition, &nwos_reference_list_class_ref))
@@ -329,11 +379,11 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	    if (use_old_decryption)
 	    {
-		assert(nwos_read_object_from_disk_and_old_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]));
+		assert(nwos_read_block_from_disk_and_old_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, seq));
 	    }
 	    else
 	    {
-		assert(nwos_read_object_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]));
+		assert(nwos_read_block_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, seq));
 	    }
 
 	    cache->num_refs = save_num_refs;
@@ -376,7 +426,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	cache->last_block = prev_ptr;        /* save it in cache in case this is the last block */
 
-	seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+	seq++;
     }
 }
 
@@ -503,7 +553,7 @@ static void write_reference_list(Block_Cache* cache)
 	    
 	    prev_ptr = next_ptr;
 
-	    seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+	    seq++;
 	}
 
 
@@ -529,7 +579,7 @@ static void write_reference_list(Block_Cache* cache)
 
 	/* printf("writing block: %02x%02x%02x%02x\n", ref.id[0], ref.id[1], ref.id[2], ref.id[3]); */
 
-	nwos_write_object_to_disk_and_encrypt(&ref, &block->dirty, FILE_BLOCK_SIZE, block->ivec, nwos_random_sequence[seq]);
+	nwos_write_block_to_disk_and_encrypt(&ref, &block->dirty, FILE_BLOCK_SIZE, block->ivec, seq);
 
 	nwos_free(block);
     }
@@ -797,7 +847,7 @@ static void verify_last_block_pointer(Block_Cache* cache, int remaining, int cal
 
 	next_ptr = next_ptr->next_block_ptr;
 
-	seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+	seq++;
 
 	running_total += MAX_REFS_IN_SIDECAR;
 
@@ -880,7 +930,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 				 (cache->num_refs + 1) * sizeof(ObjRef),               /* include void ptr or next_block_ref */
 				 cache->first_block->list.common_header.data_chksum);
 
-	    nwos_write_object_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]);
+	    nwos_write_block_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, 0);
 
 	    next_ptr = nwos_malloc(sizeof(Ref_List_Extra_Block));
 
@@ -921,7 +971,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 	next_ptr = cache->last_block;                         /* go right to the last block */
 
 	block_num = (cache->num_refs - MAX_REFS_IN_REF_LIST - 1) / MAX_REFS_IN_SIDECAR;
-	seq = (block_num + 1) % NUM_STORED_SEQ;
+	seq = block_num + 1;
 	running_total = MAX_REFS_IN_REF_LIST + block_num * MAX_REFS_IN_SIDECAR;
 
 	assert(running_total < cache->num_refs);
@@ -957,7 +1007,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 	    memcpy(ivec, next_ptr->ivec, sizeof(ivec));
 
-	    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
+	    nwos_write_block_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, seq);
 
 	    next_ptr->next_block_ptr = nwos_malloc(sizeof(Ref_List_Extra_Block));
 
@@ -992,6 +1042,8 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 }
 
 
+#ifndef PUBLIC_MODE
+
 void nwos_remove_from_references(ObjRef* ref, ObjRef* obj)
 {
     EveryObject header;
@@ -1145,7 +1197,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 				     cache->first_block->list.common_header.data_chksum);
 
 		memcpy(ivec, cache->first_block->ivec, sizeof(ivec));  /* should be all zeros */
-		nwos_write_object_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]);
+		nwos_write_block_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, 0);
 
 		memcpy(next_ptr->ivec, ivec, sizeof(ivec));
 
@@ -1195,7 +1247,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 		next_ptr = next_ptr->next_block_ptr;
 
-		seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+		seq++;
 	    }
 	}	
 
@@ -1255,7 +1307,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 					 next_ptr->checksum);
 
 		    memcpy(ivec, next_ptr->ivec, sizeof(ivec));
-		    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
+		    nwos_write_block_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, seq);
 
 		    memcpy(next_ptr->next_block_ptr->ivec, ivec, sizeof(ivec));
 
@@ -1263,7 +1315,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 		    next_ptr = next_ptr->next_block_ptr;
 
-		    seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+		    seq++;
 
 		    i = 0;
 		}
@@ -1320,7 +1372,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 		assert(cache->last_block != NULL);
 
 		block_num = (cache->num_refs - MAX_REFS_IN_REF_LIST - 1) / MAX_REFS_IN_SIDECAR;
-		seq = (block_num + 1) % NUM_STORED_SEQ;
+		seq = block_num + 1;
 		running_total = MAX_REFS_IN_REF_LIST + block_num * MAX_REFS_IN_SIDECAR;
 
 		refs_remaining = cache->num_refs - running_total;
@@ -1353,7 +1405,7 @@ void nwos_flush_dirty_ref_lists()
 	}
     }
 }
-
+#endif
 
 ReferenceList* nwos_malloc_reference_list(ObjRef* ref)
 {
