@@ -27,6 +27,23 @@
 --
 --
 -- $Log: sync_archive.c,v $
+-- Revision 1.11  2009/03/14 23:09:05  jsedwards
+-- Added includes of the new backup.h and chunk_info.h files.
+--
+-- Revision 1.10  2009/03/14 11:43:21  jsedwards
+-- Added includes for gen_id.h and user_config.h files.
+--
+-- Revision 1.9  2009/03/14 03:31:24  jsedwards
+-- Added include of bit_map.h file.
+--
+-- Revision 1.8  2009/03/13 12:36:58  jsedwards
+-- Added includes for config.h, log.h, and progress_bar.h files.  Also changed
+-- to call nwos_terminate_disk_io instead of nwos_terminate_objectify (because
+-- it calls nwos_initialize_disk_io, not nwos_initialize_objectify).
+--
+-- Revision 1.7  2009/03/08 00:06:18  jsedwards
+-- Changed include objectify_private.h to disk_io.h.
+--
 -- Revision 1.6  2009/01/02 21:42:49  jsedwards
 -- Fixed copyright year in header.  NO code changes.
 --
@@ -61,7 +78,16 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#include "objectify_private.h"
+#include "backup.h"
+#include "chunk_info.h"
+#include "bit_map.h"
+#include "config.h"
+#include "disk_io.h"
+#include "gen_id.h"
+#include "log.h"
+#include "progress_bar.h"
+#include "user_config.h"
+
 
 extern void nwos_set_private_objects_path(const char* path);
 
@@ -352,7 +378,7 @@ int main(int argc, char* argv[])
 
     close(obj_file_desc);
 
-    nwos_terminate_objectify();
+    nwos_terminate_disk_io();
 
     snprintf(msg, sizeof(msg), "Number of blocks written: %d  chunks_used: %u",
 	     num_blocks, nwos_used_private_chunks);
