@@ -29,6 +29,9 @@
 #  This script runs a quick test of file importing and exporting.
 # 
 #  $Log: test_import_cases.sh,v $
+#  Revision 1.3  2009/03/01 13:43:06  jsedwards
+#  Removed archive size parameter from prep_disk because it is no longer used.
+#
 #  Revision 1.2  2008/12/27 16:55:56  jsedwards
 #  Merged from alpha_29_5_branch back into main trunk.
 #
@@ -139,7 +142,7 @@ echo "------------------"
 
 echo $OBJECTIFY_PRIVATE_PATH
 
-if ! $EXEC/prep_disk 64M; then
+if ! $EXEC/prep_disk; then
    echo ""
    echo "prep_disk failed"
    echo ""
