@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2005-2008  QRW Software
+--                 Copyright (C) 2005-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,9 @@
 --
 --
 -- $Log: us_state.c,v $
+-- Revision 1.28  2009/03/07 20:17:19  jsedwards
+-- Removed #include of objectify_private.h, not needed.
+--
 -- Revision 1.27  2008/09/01 00:10:55  jsedwards
 -- Fix copyright year.  NO code changes.
 --
@@ -151,7 +154,6 @@
 
 #include "crc32.h"
 #include "objectify.h"
-#include "objectify_private.h"
 
 
 
